@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #include <thread>
 #include <clocale>
 #include "Config.h"
@@ -6,9 +14,10 @@
 #include "Log.h"
 
 #ifdef _WIN32
-#include "Win32/Win32Service.h"
+#include "Win32Service.h"
 #ifdef WIN32_APP
-#include "Win32/Win32App.h"
+#include <windows.h>
+#include "Win32App.h"
 #endif
 
 namespace i2p
@@ -23,6 +32,11 @@ namespace util
 		setlocale(LC_ALL, "Russian");
 		setlocale(LC_TIME, "C");
 
+		i2p::log::SetThrowFunction ([](const std::string& s)
+			{
+				MessageBox(0, TEXT(s.c_str ()), TEXT("i2pd"), MB_ICONERROR | MB_TASKMODAL | MB_OK );
+			});
+
 		if (!Daemon_Singleton::init(argc, argv))
 			return false;
 
