@@ -1,10 +1,18 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #ifdef _WIN32
 #define _CRT_SECURE_NO_WARNINGS // to use freopen
 #endif
 
 #include "Win32Service.h"
 #include <assert.h>
-#include <strsafe.h>
+//#include <strsafe.h>
 #include <windows.h>
 
 #include "Daemon.h"
