@@ -19,7 +19,7 @@
 #include "version.h"
 #include "Log.h"
 #include "Family.h"
-#include "TunnelConfig.h"
+#include "ECIESX25519AEADRatchetSession.h"
 #include "RouterContext.h"
 
 namespace i2p
@@ -28,7 +28,7 @@ namespace i2p
 
 	RouterContext::RouterContext ():
 		m_LastUpdateTime (0), m_AcceptsTunnels (true), m_IsFloodfill (false),
-		m_ShareRatio (100), m_Status (eRouterStatusOK),
+		m_ShareRatio (100), m_Status (eRouterStatusUnknown),
 		m_Error (eRouterErrorNone), m_NetID (I2PD_NET_ID)
 	{
 	}
@@ -45,15 +45,15 @@ namespace i2p
 		if (IsECIES ())
 		{	
 			auto initState = new i2p::crypto::NoiseSymmetricState ();
-			i2p::tunnel::InitBuildRequestRecordNoiseState (*initState);
-			initState->MixHash (GetIdentity ()->GetEncryptionPublicKey (), 32); // h = SHA256(h || hepk)
+			i2p::crypto::InitNoiseNState (*initState, GetIdentity ()->GetEncryptionPublicKey ());
 			m_InitialNoiseState.reset (initState);	
 		}	
 	}
 
 	void RouterContext::CreateNewRouter ()
 	{
-		m_Keys = i2p::data::PrivateKeys::CreateRandomKeys (i2p::data::SIGNING_KEY_TYPE_EDDSA_SHA512_ED25519);
+		m_Keys = i2p::data::PrivateKeys::CreateRandomKeys (i2p::data::SIGNING_KEY_TYPE_EDDSA_SHA512_ED25519,
+			i2p::data::CRYPTO_KEY_TYPE_ECIES_X25519_AEAD);
 		SaveKeys ();
 		NewRouterInfo ();
 	}
@@ -72,10 +72,12 @@ namespace i2p
 		bool ipv6;           i2p::config::GetOption("ipv6", ipv6);
 		bool ssu;            i2p::config::GetOption("ssu", ssu);
 		bool ntcp2;          i2p::config::GetOption("ntcp2.enabled", ntcp2);
+		bool ygg; 			 i2p::config::GetOption("meshnets.yggdrasil", ygg);
 		bool nat;            i2p::config::GetOption("nat", nat);
 		std::string ifname;  i2p::config::GetOption("ifname", ifname);
 		std::string ifname4; i2p::config::GetOption("ifname4", ifname4);
 		std::string ifname6; i2p::config::GetOption("ifname6", ifname6);
+		uint8_t caps = 0;
 		if (ipv4)
 		{
 			std::string host = "127.0.0.1";
@@ -89,7 +91,10 @@ namespace i2p
 				host = i2p::util::net::GetInterfaceAddress(ifname4, false).to_string();
 
 			if (ssu)
+			{	
 				routerInfo.AddSSUAddress (host.c_str(), port, nullptr);
+				caps |= i2p::data::RouterInfo::eReachable | i2p::data::RouterInfo::eSSUTesting | i2p::data::RouterInfo::eSSUIntroducer; // R, BC
+			}	
 		}
 		if (ipv6)
 		{
@@ -103,11 +108,13 @@ namespace i2p
 				host = i2p::util::net::GetInterfaceAddress(ifname6, true).to_string();
 
 			if (ssu)
+			{	
 				routerInfo.AddSSUAddress (host.c_str(), port, nullptr);
+				caps |= i2p::data::RouterInfo::eReachable; // R
+			}	
 		}
-
-		routerInfo.SetCaps (i2p::data::RouterInfo::eReachable |
-			i2p::data::RouterInfo::eSSUTesting | i2p::data::RouterInfo::eSSUIntroducer); // LR, BC
+		
+		routerInfo.SetCaps (caps); // caps + L 
 		routerInfo.SetProperty ("netId", std::to_string (m_NetID));
 		routerInfo.SetProperty ("router.version", I2P_VERSION);
 		routerInfo.CreateBuffer (m_Keys);
@@ -117,11 +124,12 @@ namespace i2p
 		if (ntcp2) // we don't store iv in the address if non published so we must update it from keys
 		{
 			if (!m_NTCP2Keys) NewNTCP2Keys ();
-			UpdateNTCP2Address (true);
 			bool published; i2p::config::GetOption("ntcp2.published", published);
+			if (ipv4 || !published) UpdateNTCP2Address (true); // create not published NTCP2 address
 			if (published)
 			{
-				PublishNTCP2Address (port, true);
+				if (ipv4)
+					PublishNTCP2Address (port, true);
 				if (ipv6)
 				{
 					// add NTCP2 ipv6 address
@@ -131,7 +139,21 @@ namespace i2p
 					m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, boost::asio::ip::address_v6::from_string (host), port);
 				}
 			}
+			// enable added NTCP2 addresses
+			if (ipv4) m_RouterInfo.EnableV4 ();
+			if (ipv6) m_RouterInfo.EnableV6 ();
 		}
+		if (ygg)
+		{
+			auto yggaddr = i2p::util::net::GetYggdrasilAddress ();
+			if (!yggaddr.is_unspecified ())
+			{	
+				if (!m_NTCP2Keys) NewNTCP2Keys ();
+				m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, yggaddr, port);
+				m_RouterInfo.EnableMesh ();
+				UpdateRouterInfo ();
+			}	
+		}	
 	}
 
 	void RouterContext::UpdateRouterInfo ()
@@ -245,7 +267,8 @@ namespace i2p
 		bool updated = false;
 		for (auto& address : m_RouterInfo.GetAddresses ())
 		{
-			if (address->host != host && address->IsCompatible (host))
+			if (address->host != host && address->IsCompatible (host) && 
+			    !i2p::util::net::IsYggdrasilAddress (address->host))
 			{
 				address->host = host;
 				if (host.is_v6 () && address->transportStyle == i2p::data::RouterInfo::eTransportSSU)
@@ -521,15 +544,44 @@ namespace i2p
 		UpdateRouterInfo ();
 	}
 
+	void RouterContext::SetSupportsMesh (bool supportsmesh, const boost::asio::ip::address_v6& host)
+	{	
+		if (supportsmesh)
+		{	
+			m_RouterInfo.EnableMesh ();
+			uint16_t port = 0;
+			i2p::config::GetOption ("ntcp2.port", port);
+			if (!port) i2p::config::GetOption("port", port);
+			bool foundMesh = false;
+			auto& addresses = m_RouterInfo.GetAddresses ();
+			for (auto& addr: addresses)
+			{
+				if (!port) port = addr->port;
+				if (i2p::util::net::IsYggdrasilAddress (addr->host))
+				{
+					foundMesh = true;
+					break;
+				}	
+			}
+			if (!foundMesh)
+				m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, host, port);
+		}	
+		else
+			m_RouterInfo.DisableMesh ();
+		UpdateRouterInfo ();
+	}
+		
 	void RouterContext::UpdateNTCP2V6Address (const boost::asio::ip::address& host)
 	{
+		bool isYgg = i2p::util::net::IsYggdrasilAddress (host);
 		bool updated = false;
 		auto& addresses = m_RouterInfo.GetAddresses ();
 		for (auto& addr: addresses)
 		{
 			if (addr->IsPublishedNTCP2 ())
 			{
-				if (addr->host.is_v6 ())
+				bool isYgg1 = i2p::util::net::IsYggdrasilAddress (addr->host);
+				if (addr->host.is_v6 () && ((isYgg && isYgg1) || (!isYgg && !isYgg1)))
 				{
 					if (addr->host != host)
 					{
@@ -591,7 +643,8 @@ namespace i2p
 			// update keys
 			LogPrint (eLogInfo, "Router: router keys are obsolete. Creating new");
 			oldIdentity = m_Keys.GetPublic ();
-			m_Keys = i2p::data::PrivateKeys::CreateRandomKeys (i2p::data::SIGNING_KEY_TYPE_EDDSA_SHA512_ED25519);
+			m_Keys = i2p::data::PrivateKeys::CreateRandomKeys (i2p::data::SIGNING_KEY_TYPE_EDDSA_SHA512_ED25519,
+				i2p::data::CRYPTO_KEY_TYPE_ECIES_X25519_AEAD);
 			SaveKeys ();
 		}	
 		// read NTCP2 keys if available
@@ -630,7 +683,8 @@ namespace i2p
 
 		// read NTCP2
 		bool ntcp2;  i2p::config::GetOption("ntcp2.enabled", ntcp2);
-		if (ntcp2)
+		bool ygg;  i2p::config::GetOption("meshnets.yggdrasil", ygg);
+		if (ntcp2 || ygg)
 		{
 			if (!m_NTCP2Keys) NewNTCP2Keys ();
 			UpdateNTCP2Address (true); // enable NTCP2
@@ -674,7 +728,21 @@ namespace i2p
 	void RouterContext::ProcessGarlicMessage (std::shared_ptr<I2NPMessage> msg)
 	{
 		std::unique_lock<std::mutex> l(m_GarlicMutex);
-		i2p::garlic::GarlicDestination::ProcessGarlicMessage (msg);
+		if (IsECIES ())
+		{
+			uint8_t * buf = msg->GetPayload ();
+			uint32_t len = bufbe32toh (buf);
+			if (len > msg->GetLength ())
+			{
+				LogPrint (eLogWarning, "Router: garlic message length ", len, " exceeds I2NP message length ", msg->GetLength ());
+				return;
+			}
+			buf += 4;
+			auto session = std::make_shared<i2p::garlic::ECIESX25519AEADRatchetSession>(this, false);
+			session->HandleNextMessageForRouter (buf, len);
+		}	
+		else	
+			i2p::garlic::GarlicDestination::ProcessGarlicMessage (msg);
 	}
 
 	void RouterContext::ProcessDeliveryStatusMessage (std::shared_ptr<I2NPMessage> msg)
@@ -714,7 +782,11 @@ namespace i2p
 			m_CurrentNoiseState.reset (new i2p::crypto::NoiseSymmetricState (*m_InitialNoiseState));		
 			m_CurrentNoiseState->MixHash (encrypted, 32); // h = SHA256(h || sepk)
 			uint8_t sharedSecret[32];
-			m_Decryptor->Decrypt (encrypted, sharedSecret, ctx, false);
+			if (!m_Decryptor->Decrypt (encrypted, sharedSecret, ctx, false))
+			{
+				LogPrint (eLogWarning, "Router: Incorrect ephemeral public key");
+				return false;
+			}	
 			m_CurrentNoiseState->MixKey (sharedSecret); 
 			encrypted += 32;
 			uint8_t nonce[12];
