@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #ifndef SSU_H__
 #define SSU_H__
 
@@ -40,7 +48,7 @@ namespace transport
 		public:
 
 			SSUServer (int port);
-			SSUServer (const boost::asio::ip::address & addr, int port);			// ipv6 only constructor
+			SSUServer (const boost::asio::ip::address & addr, int port); // ipv6 only constructor
 			~SSUServer ();
 			void Start ();
 			void Stop ();
@@ -56,7 +64,6 @@ namespace transport
 			void DeleteAllSessions ();
 
 			boost::asio::io_service& GetService () { return m_Service; };
-			boost::asio::io_service& GetServiceV6 () { return m_ServiceV6; };
 			const boost::asio::ip::udp::endpoint& GetEndpoint () const { return m_Endpoint; };
 			void Send (const uint8_t * buf, size_t len, const boost::asio::ip::udp::endpoint& to);
 			void AddRelay (uint32_t tag, std::shared_ptr<SSUSession> relay);
@@ -74,7 +81,6 @@ namespace transport
 			void OpenSocket ();
 			void OpenSocketV6 ();
 			void Run ();
-			void RunV6 ();
 			void RunReceivers ();
 			void RunReceiversV6 ();
 			void Receive ();
@@ -113,10 +119,10 @@ namespace transport
 			};
 
 			bool m_OnlyV6;
-			bool m_IsRunning;
-			std::thread * m_Thread, * m_ThreadV6, * m_ReceiversThread, * m_ReceiversThreadV6;
-			boost::asio::io_service m_Service, m_ServiceV6, m_ReceiversService, m_ReceiversServiceV6;
-			boost::asio::io_service::work m_Work, m_WorkV6, m_ReceiversWork, m_ReceiversWorkV6;
+			volatile bool m_IsRunning;
+			std::thread * m_Thread, * m_ReceiversThread, * m_ReceiversThreadV6;
+			boost::asio::io_service m_Service, m_ReceiversService, m_ReceiversServiceV6;
+			boost::asio::io_service::work m_Work, m_ReceiversWork, m_ReceiversWorkV6;
 			boost::asio::ip::udp::endpoint m_Endpoint, m_EndpointV6;
 			boost::asio::ip::udp::socket m_Socket, m_SocketV6;
 			boost::asio::deadline_timer m_IntroducersUpdateTimer, m_PeerTestsCleanupTimer,
@@ -135,4 +141,3 @@ namespace transport
 }
 
 #endif
-
