@@ -18,6 +18,7 @@
 #include "Log.h"
 #include "I2PEndian.h"
 #include "Timestamp.h"
+#include "util.h"
 
 #ifdef _WIN32
 	#ifndef _WIN64
@@ -148,6 +149,8 @@ namespace util
 
 	void NTPTimeSync::Run ()
 	{
+		i2p::util::SetThreadName("Timesync");
+
 		while (m_IsRunning)
 		{
 			try
