@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #include "I2PEndian.h"
 #include <string.h>
 #include "Crypto.h"
@@ -64,8 +72,7 @@ namespace tunnel
 							m.hash = i2p::data::IdentHash (fragment);
 							fragment += 32; // to hash
 						break;
-						default:
-							;
+						default: ;
 					}
 
 					bool isFragmented = flag & 0x08;
