@@ -29,6 +29,10 @@ namespace tunnel
 		m_NumInboundTunnels (numInboundTunnels), m_NumOutboundTunnels (numOutboundTunnels),
 		m_IsActive (true), m_CustomPeerSelector(nullptr)
 	{
+		if (m_NumInboundTunnels > TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY) 
+			m_NumInboundTunnels = TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY;
+		if (m_NumOutboundTunnels > TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY) 
+			m_NumOutboundTunnels = TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY;	
 		m_NextManageTime = i2p::util::GetSecondsSinceEpoch () + rand () % TUNNEL_POOL_MANAGE_INTERVAL;
 	}
 
@@ -235,7 +239,7 @@ namespace tunnel
 			for (const auto& it : m_InboundTunnels)
 				if (it->IsEstablished ()) num++;
 		}
-		if (!num && !m_OutboundTunnels.empty ())
+		if (!num && !m_OutboundTunnels.empty () && m_NumOutboundHops > 0)
 		{
 			for (auto it: m_OutboundTunnels)
 			{	
@@ -414,7 +418,8 @@ namespace tunnel
 		else if (i2p::transport::transports.GetNumPeers () > 25)
 		{
 			auto r = i2p::transport::transports.GetRandomPeer ();
-			if (r && !r->GetProfile ()->IsBad ())
+			if (r && !r->GetProfile ()->IsBad () && 
+				(numHops > 1 || !inbound || r->IsReachable ())) // first must be reachable
 			{
 				prevHop = r;
 				peers.push_back (r->GetRouterIdentity ());
@@ -430,6 +435,12 @@ namespace tunnel
 				LogPrint (eLogError, "Tunnels: Can't select next hop for ", prevHop->GetIdentHashBase64 ());
 				return false;
 			}
+			if (inbound && (i == numHops - 1) && !hop->IsReachable ())
+			{
+				// if first is not reachable try again
+				auto hop1 = nextHop (prevHop);
+				if (hop1) hop = hop1;
+			}	
 			prevHop = hop;
 			peers.push_back (hop->GetRouterIdentity ());
 		}
@@ -559,7 +570,7 @@ namespace tunnel
 			{
 				config = std::make_shared<TunnelConfig>(tunnel->GetPeers (), inboundTunnel->GetNextTunnelID (), inboundTunnel->GetNextIdentHash ());
 			}
-			if(m_NumOutboundHops == 0 || config)
+			if (!m_NumOutboundHops || config)
 			{
 				auto newTunnel = tunnels.CreateOutboundTunnel (config);
 				newTunnel->SetTunnelPool (shared_from_this ());
@@ -574,8 +585,12 @@ namespace tunnel
 	void TunnelPool::CreatePairedInboundTunnel (std::shared_ptr<OutboundTunnel> outboundTunnel)
 	{
 		LogPrint (eLogDebug, "Tunnels: Creating paired inbound tunnel...");
-		auto tunnel = tunnels.CreateInboundTunnel (std::make_shared<TunnelConfig>(outboundTunnel->GetInvertedPeers ()), outboundTunnel);
+		auto tunnel = tunnels.CreateInboundTunnel (
+			m_NumOutboundHops > 0 ? std::make_shared<TunnelConfig>(outboundTunnel->GetInvertedPeers ()) : nullptr, 
+		    outboundTunnel);
 		tunnel->SetTunnelPool (shared_from_this ());
+		if (tunnel->IsEstablished ()) // zero hops
+			TunnelCreated (tunnel);
 	}
 
 	void TunnelPool::SetCustomPeerSelector(ITunnelPeerSelector * selector)
