@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #ifndef TUNNEL_POOL__
 #define TUNNEL_POOL__
 
@@ -19,17 +27,14 @@ namespace i2p
 {
 namespace tunnel
 {
+	const int TUNNEL_POOL_MANAGE_INTERVAL = 10; // in seconds
+	const int TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY = 16;
+	const int TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY = 16;
+	
 	class Tunnel;
 	class InboundTunnel;
 	class OutboundTunnel;
 
-
-	enum TunnelBuildResult {
-		eBuildResultOkay, // tunnel was built okay
-		eBuildResultRejected, // tunnel build was explicitly rejected
-		eBuildResultTimeout // tunnel build timed out
-	};
-
 	typedef std::shared_ptr<const i2p::data::IdentityEx> Peer;
 	typedef std::vector<Peer> Path;
 
@@ -38,7 +43,6 @@ namespace tunnel
 	{
 		virtual ~ITunnelPeerSelector() {};
 		virtual bool SelectPeers(Path & peers, int hops, bool isInbound) = 0;
-		virtual bool OnBuildResult(const Path & peers, bool isInbound, TunnelBuildResult result) = 0;
 	};
 
 
@@ -69,6 +73,7 @@ namespace tunnel
 			std::shared_ptr<InboundTunnel> GetNextInboundTunnel (std::shared_ptr<InboundTunnel> excluded = nullptr) const;
 			std::shared_ptr<OutboundTunnel> GetNewOutboundTunnel (std::shared_ptr<OutboundTunnel> old) const;
 			void TestTunnels ();
+			void ManageTunnels (uint64_t ts);
 			void ProcessGarlicMessage (std::shared_ptr<I2NPMessage> msg);
 			void ProcessDeliveryStatus (std::shared_ptr<I2NPMessage> msg);
 
@@ -83,25 +88,23 @@ namespace tunnel
 
 			/** i2cp reconfigure */
 			bool Reconfigure(int inboundHops, int outboundHops, int inboundQuant, int outboundQuant);
-    
+
 			void SetCustomPeerSelector(ITunnelPeerSelector * selector);
 			void UnsetCustomPeerSelector();
 			bool HasCustomPeerSelector();
 
-		/** @brief make this tunnel pool yield tunnels that fit latency range [min, max] */
-		void RequireLatency(uint64_t min, uint64_t max) { m_MinLatency = min; m_MaxLatency = max; }
+			/** @brief make this tunnel pool yield tunnels that fit latency range [min, max] */
+			void RequireLatency(uint64_t min, uint64_t max) { m_MinLatency = min; m_MaxLatency = max; }
 
-		/** @brief return true if this tunnel pool has a latency requirement */
-		bool HasLatencyRequirement() const { return m_MinLatency > 0 && m_MaxLatency > 0; }
+			/** @brief return true if this tunnel pool has a latency requirement */
+			bool HasLatencyRequirement() const { return m_MinLatency > 0 && m_MaxLatency > 0; }
 
-		/** @brief get the lowest latency tunnel in this tunnel pool regardless of latency requirements */
-		std::shared_ptr<InboundTunnel> GetLowestLatencyInboundTunnel(std::shared_ptr<InboundTunnel> exclude=nullptr) const;
-		std::shared_ptr<OutboundTunnel> GetLowestLatencyOutboundTunnel(std::shared_ptr<OutboundTunnel> exclude=nullptr) const;
+			/** @brief get the lowest latency tunnel in this tunnel pool regardless of latency requirements */
+			std::shared_ptr<InboundTunnel> GetLowestLatencyInboundTunnel(std::shared_ptr<InboundTunnel> exclude = nullptr) const;
+			std::shared_ptr<OutboundTunnel> GetLowestLatencyOutboundTunnel(std::shared_ptr<OutboundTunnel> exclude = nullptr) const;
 
-		void OnTunnelBuildResult(std::shared_ptr<Tunnel> tunnel, TunnelBuildResult result);
-
-		// for overriding tunnel peer selection
-		std::shared_ptr<const i2p::data::RouterInfo> SelectNextHop (std::shared_ptr<const i2p::data::RouterInfo> prevHop) const;
+			// for overriding tunnel peer selection
+			std::shared_ptr<const i2p::data::RouterInfo> SelectNextHop (std::shared_ptr<const i2p::data::RouterInfo> prevHop) const;
 
 		private:
 
@@ -125,11 +128,12 @@ namespace tunnel
 			mutable std::mutex m_TestsMutex;
 			std::map<uint32_t, std::pair<std::shared_ptr<OutboundTunnel>, std::shared_ptr<InboundTunnel> > > m_Tests;
 			bool m_IsActive;
+			uint64_t m_NextManageTime; // in seconds
 			std::mutex m_CustomPeerSelectorMutex;
 			ITunnelPeerSelector * m_CustomPeerSelector;
 
-		uint64_t m_MinLatency=0; // if > 0 this tunnel pool will try building tunnels with minimum latency by ms
-		uint64_t m_MaxLatency=0; // if > 0 this tunnel pool will try building tunnels with maximum latency by ms
+			uint64_t m_MinLatency = 0; // if > 0 this tunnel pool will try building tunnels with minimum latency by ms
+			uint64_t m_MaxLatency = 0; // if > 0 this tunnel pool will try building tunnels with maximum latency by ms
 
 		public:
 
