@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #include "Destination.h"
 #include "Identity.h"
 #include "ClientContext.h"
@@ -84,7 +92,7 @@ namespace client
 			auto itr = m_ReadyCallbacks.begin();
 			while(itr != m_ReadyCallbacks.end())
 			{
-			    if(itr->second != NEVER_TIMES_OUT && now >= itr->second)
+				if(itr->second != NEVER_TIMES_OUT && now >= itr->second)
 				{
 					itr->first(boost::asio::error::timed_out);
 					itr = m_ReadyCallbacks.erase(itr);
@@ -94,16 +102,16 @@ namespace client
 			}
 		}
 		if(!ec && m_ReadyCallbacks.size())
-		    TriggerReadyCheckTimer();
+			TriggerReadyCheckTimer();
 		else
-		    m_ReadyTimerTriggered = false;
+			m_ReadyTimerTriggered = false;
 	}
 
 	void I2PService::CreateStream (StreamRequestComplete streamRequestComplete, const std::string& dest, int port) {
 		assert(streamRequestComplete);
-		i2p::data::IdentHash identHash;
-		if (i2p::client::context.GetAddressBook ().GetIdentHash (dest, identHash))
-			CreateStream(streamRequestComplete, identHash, port);
+		auto address = i2p::client::context.GetAddressBook ().GetAddress (dest);
+		if (address)
+			CreateStream(streamRequestComplete, address, port);
 		else
 		{
 			LogPrint (eLogWarning, "I2PService: Remote destination not found: ", dest);
@@ -111,27 +119,33 @@ namespace client
 		}
 	}
 
-	void I2PService::CreateStream(StreamRequestComplete streamRequestComplete, const i2p::data::IdentHash & identHash, int port)
+	void I2PService::CreateStream(StreamRequestComplete streamRequestComplete, std::shared_ptr<const Address> address, int port)
 	{
-		if(m_ConnectTimeout)
+		if(m_ConnectTimeout && !m_LocalDestination->IsReady())
 		{
-			if(m_LocalDestination->IsReady())
-				m_LocalDestination->CreateStream (streamRequestComplete, identHash, port);
-			else
-			{
-				AddReadyCallback([this, streamRequestComplete, identHash, port] (const boost::system::error_code & ec) {
-						if(ec)
-						{
-							LogPrint(eLogWarning, "I2PService::CeateStream() ", ec.message());
-							streamRequestComplete(nullptr);
-						}
+			AddReadyCallback([this, streamRequestComplete, address, port] (const boost::system::error_code & ec)
+				{
+					if(ec)
+					{
+						LogPrint(eLogWarning, "I2PService::CreateStream() ", ec.message());
+						streamRequestComplete(nullptr);
+					}
+					else
+					{
+						if (address->IsIdentHash ())
+							this->m_LocalDestination->CreateStream(streamRequestComplete, address->identHash, port);
 						else
-							this->m_LocalDestination->CreateStream(streamRequestComplete, identHash, port);
-					});
-			}
+							this->m_LocalDestination->CreateStream (streamRequestComplete, address->blindedPublicKey, port);
+					}
+				});
 		}
 		else
-			m_LocalDestination->CreateStream(streamRequestComplete, identHash, port);
+		{
+			if (address->IsIdentHash ())
+				m_LocalDestination->CreateStream (streamRequestComplete, address->identHash, port);
+			else
+				m_LocalDestination->CreateStream (streamRequestComplete, address->blindedPublicKey, port);
+		}
 	}
 
 	TCPIPPipe::TCPIPPipe(I2PService * owner, std::shared_ptr<boost::asio::ip::tcp::socket> upstream, std::shared_ptr<boost::asio::ip::tcp::socket> downstream) : I2PServiceHandler(owner), m_up(upstream), m_down(downstream)
@@ -176,7 +190,7 @@ namespace client
 		{
 			m_up->async_read_some(boost::asio::buffer(m_upstream_to_down_buf, TCP_IP_PIPE_BUFFER_SIZE),
 				std::bind(&TCPIPPipe::HandleUpstreamReceived, shared_from_this(),
-				std::placeholders::_1, std::placeholders::_2));
+					std::placeholders::_1, std::placeholders::_2));
 		}
 		else
 			LogPrint(eLogError, "TCPIPPipe: upstream receive: no socket");
@@ -187,7 +201,7 @@ namespace client
 		if (m_down) {
 			m_down->async_read_some(boost::asio::buffer(m_downstream_to_up_buf, TCP_IP_PIPE_BUFFER_SIZE),
 				std::bind(&TCPIPPipe::HandleDownstreamReceived, shared_from_this(),
-				std::placeholders::_1, std::placeholders::_2));
+					std::placeholders::_1, std::placeholders::_2));
 		}
 		else
 			LogPrint(eLogError, "TCPIPPipe: downstream receive: no socket");
@@ -201,8 +215,8 @@ namespace client
 			boost::asio::async_write(*m_up, boost::asio::buffer(m_upstream_buf, len),
 				boost::asio::transfer_all(),
 				std::bind(&TCPIPPipe::HandleUpstreamWrite,
-				shared_from_this(),
-				std::placeholders::_1));
+					shared_from_this(),
+					std::placeholders::_1));
 		}
 		else
 			LogPrint(eLogError, "TCPIPPipe: upstream write: no socket");
@@ -216,8 +230,8 @@ namespace client
 			boost::asio::async_write(*m_down, boost::asio::buffer(m_downstream_buf, len),
 				boost::asio::transfer_all(),
 				std::bind(&TCPIPPipe::HandleDownstreamWrite,
-				shared_from_this(),
-				std::placeholders::_1));
+					shared_from_this(),
+					std::placeholders::_1));
 		}
 		else
 			LogPrint(eLogError, "TCPIPPipe: downstream write: no socket");
@@ -279,7 +293,7 @@ namespace client
 	void TCPIPAcceptor::Start ()
 	{
 		m_Acceptor.reset (new boost::asio::ip::tcp::acceptor (GetService (), m_LocalEndpoint));
-		//update the local end point in case port has been set zero and got updated now
+		// update the local end point in case port has been set zero and got updated now
 		m_LocalEndpoint = m_Acceptor->local_endpoint();
 		m_Acceptor->listen ();
 		Accept ();
