@@ -12,6 +12,7 @@
 #include "Destination.h"
 #include "ClientContext.h"
 #include "I2PTunnel.h"
+#include "util.h"
 
 namespace i2p
 {
@@ -862,7 +863,7 @@ namespace client
 			std::placeholders::_3, std::placeholders::_4,
 			std::placeholders::_5));
 		dgram->SetRawReceiver(std::bind(&I2PUDPClientTunnel::HandleRecvFromI2PRaw, this,
-			std::placeholders::_1, std::placeholders::_2, std::placeholders::_3, std::placeholders::_4));	
+			std::placeholders::_1, std::placeholders::_2, std::placeholders::_3, std::placeholders::_4));
 	}
 
 	void I2PUDPClientTunnel::Start() {
@@ -891,11 +892,11 @@ namespace client
 		}
 		auto remotePort = m_RecvEndpoint.port();
 		if (!m_LastPort || m_LastPort != remotePort)
-		{	
+		{
 			auto itr = m_Sessions.find(remotePort);
-			if (itr != m_Sessions.end()) 
+			if (itr != m_Sessions.end())
 				m_LastSession = itr->second;
-			else	
+			else
 			{
 				m_LastSession = std::make_shared<UDPConvo>(boost::asio::ip::udp::endpoint(m_RecvEndpoint), 0);
 				m_Sessions.emplace (remotePort, m_LastSession);
@@ -940,6 +941,7 @@ namespace client
 	}
 
 	void I2PUDPClientTunnel::TryResolving() {
+		i2p::util::SetThreadName("UDP Resolver");
 		LogPrint(eLogInfo, "UDP Tunnel: Trying to resolve ", m_RemoteDest);
 
 		std::shared_ptr<const Address> addr;
