@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #include "MatchedDestination.h"
 #include "Log.h"
 #include "ClientContext.h"
@@ -8,16 +16,16 @@ namespace i2p
 namespace client
 {
 	MatchedTunnelDestination::MatchedTunnelDestination(const i2p::data::PrivateKeys & keys, const std::string & remoteName, const std::map<std::string, std::string> * params)
-		: ClientDestination(keys, false, params),
+		: RunnableClientDestination(keys, false, params),
 			m_RemoteName(remoteName) {}
 
 
 	void MatchedTunnelDestination::ResolveCurrentLeaseSet()
 	{
-		auto addr = i2p::client::context.GetAddressBook().GetAddress (m_RemoteName);	
+		auto addr = i2p::client::context.GetAddressBook().GetAddress (m_RemoteName);
 		if(addr && addr->IsIdentHash ())
 		{
-			m_RemoteIdent = addr->identHash;	
+			m_RemoteIdent = addr->identHash;
 			auto ls = FindLeaseSet(m_RemoteIdent);
 			if(ls)
 				HandleFoundCurrentLeaseSet(ls);
@@ -39,35 +47,25 @@ namespace client
 		{
 			m_ResolveTimer->expires_from_now(boost::posix_time::seconds(1));
 			m_ResolveTimer->async_wait([&](const boost::system::error_code & ec) {
-					if(!ec)	ResolveCurrentLeaseSet();
+				if(!ec)	ResolveCurrentLeaseSet();
 			});
 		}
 	}
 
 
-	bool MatchedTunnelDestination::Start()
+	void MatchedTunnelDestination::Start()
 	{
-		if(ClientDestination::Start())
-		{
-			m_ResolveTimer = std::make_shared<boost::asio::deadline_timer>(GetService());
-			GetTunnelPool()->SetCustomPeerSelector(this);
-			ResolveCurrentLeaseSet();
-			return true;
-		}
-		else
-			return false;
+		ClientDestination::Start();
+		m_ResolveTimer = std::make_shared<boost::asio::deadline_timer>(GetService());
+		GetTunnelPool()->SetCustomPeerSelector(this);
+		ResolveCurrentLeaseSet();
 	}
 
-	bool MatchedTunnelDestination::Stop()
+	void MatchedTunnelDestination::Stop()
 	{
-		if(ClientDestination::Stop())
-		{
-			if(m_ResolveTimer)
-				m_ResolveTimer->cancel();
-			return true;
-		}
-		else
-			return false;
+		ClientDestination::Stop();
+		if(m_ResolveTimer)
+			m_ResolveTimer->cancel();
 	}
 
 
@@ -104,10 +102,5 @@ namespace client
 		}
 		return true;
 	}
-
-	bool MatchedTunnelDestination::OnBuildResult(const i2p::tunnel::Path & path, bool inbound, i2p::tunnel::TunnelBuildResult result)
-	{
-		return true;
-	}
 }
 }
